import java.util.ArrayList;
import java.util.Arrays;

/**
 * Die Klasse TSP.
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class TSP
{
    private Graph graph;
    private Knoten start;
    private int anzahlKnoten;
    private Weg besterRundweg = null;

    /**
     * Konstruktor für Objekte der Klasse TSP
     */
    public TSP(int anzahlKnoten)
    {
        this.anzahlKnoten = anzahlKnoten;
        graph = new Graph(anzahlKnoten, 500,500);
        graph.vollstaendig();
        graph.zeichne();
    }

    /**
     * definiert einen vorzugebenden Knoten als Startknoten,
     * i<=anzahl !
     * und startet dann die Suche aller Rundwege
     */
    public void starteSucheMit(int i)
    {
        start = Knoten.gibKnoten(i);
        schritt(new Weg(start));
        if (besterRundweg!=null) besterRundweg.zeichneWeg();
    }

    /**
     * fuehrt den Schritt in die Tiefe aus
     */
    public void schritt(Weg weg)
    {
        //weg.zeigeWeg();
        if ((besterRundweg!=null)&&
        (weg.wegLaenge()>=besterRundweg.wegLaenge())) return;
        if (weg.istRundweg()) // neuer Rundweg gefunden
        {
            if ((besterRundweg==null)||
            (weg.wegLaenge()<besterRundweg.wegLaenge()))
            {
                besterRundweg=weg;
                besterRundweg.zeigeWeg();
            }
            return;
        }
        else if (weg.keinZyklus())
        {
            Knoten knoten = weg.gibEndKnoten();
            for (Kante kante : knoten.gibKanten())
                schritt(new Weg(weg, kante));
        }
        else return;
    }

    ///// Interne Klasse Weg /////
    private class Weg
    implements Comparable
    {        
        private Kante[] kanten;
        private Knoten[] knoten;
        private Knoten endKnoten;
        private int anzahlKanten;
        private double wegLaenge;
        private boolean keinZyklus;

        /**
         * Konstruktor fuer den Start
         */
        public Weg(Knoten startKnoten)
        {
            kanten = new Kante[0];
            knoten = new Knoten[1];
            knoten[0]=startKnoten;
            endKnoten = startKnoten;
            wegLaenge = 0;
            keinZyklus = true;
        }

        /**
         * Konstruktor fuer einen fortsetzenden Weg
         */
        public Weg(Weg weg, Kante neueKante)
        {
            //kanten = new Kante[weg.gibKanten().length+1];
            kanten = Arrays.copyOf(weg.gibKanten(),weg.gibKanten().length+1);
            //knoten = Kante[weg.gibKanten().length+2];
            knoten = Arrays.copyOf(weg.gibKnoten(),kanten.length+1);
            wegLaenge = 0;
            keinZyklus = true;
            // Kopie fertig
            kanten[kanten.length-1]=neueKante;
            wegLaenge = weg.wegLaenge() + neueKante.laenge();
            endKnoten = neueKante.andererKnoten(weg.gibEndKnoten());
            for (int i=0;i<knoten.length-1;i++)
            if (knoten[i]==endKnoten) keinZyklus = false;
            knoten[knoten.length-1]=endKnoten;
        }

        /**
         * get-Methode fuer die Kanten
         */
        public Kante[] gibKanten() { return kanten; }

        /**
         * get-Methode fuer die Knoten
         */
        public Knoten[] gibKnoten() { return knoten; }

        /**
         * get-Methode fuer die Anzahl der Kanten
         */
        public int gibAnzahlKanten() { return anzahlKanten; }

        /**
         * get-Methode fuer den Endknoten
         */
        public Knoten gibEndKnoten() { return endKnoten; }

        // Fuer das Interface  Comparable notwendige Methode:
        public int compareTo(Object obj) {
            if (wegLaenge()<((Weg)obj).wegLaenge()) return -1;
            else if (wegLaenge()>((Weg)obj).wegLaenge()) return 1;
            else return 0;
        }

        /**
         * get-Methode von zyklus
         */
        public boolean keinZyklus() { return keinZyklus; }

        /**
         * gibt die Weglaenge zurueck
         */
        public double wegLaenge() { return wegLaenge; }

        /**
         * gibt zurueck, ob der Weg ein Rundweg ist
         */
        public boolean istRundweg()
        {
            return (endKnoten==start)&&(kanten.length==anzahlKnoten);
        }

        /**
         * Anzeige in der Konsole
         */
        public void zeigeWeg()
        {
            System.out.print(start.gibNamen());
            Knoten aktuell = start;
            for (Kante kante : kanten)
            {
                aktuell = kante.andererKnoten(aktuell);
                System.out.print(" -> "+aktuell.gibNamen());
            }
            System.out.println(", Laenge "+wegLaenge());
        }

        /**
         * Anzeige im Bild
         */
        public void zeichneWeg()
        {
            Knoten aktuell = start;
            for (Kante kante : kanten)
            {
                aktuell = kante.andererKnoten(aktuell);
                kante.zeichne("rot");
            }
        }

    }
}
